
namespace angGoGo.GalleryNET.Core
{
	using System;
	using System.Web;
	using System.Drawing;
	using System.IO;
	using System.Collections;
	using angGoGo.GalleryNET.SystemFramework;
	using angGoGo.GalleryNET.Template;
	using angGoGo.GalleryNET.GalleryData;

	/// <summary>
	/// To show the folders and albums to the webpage in the gallery.
	///		<remark>
	///			Actually, GalleryGrid is a big TABLE. It contains folders and albums.
	///			
	///			Folders
	///				They are folders under the base folder. 
	///				
	///			Albums
	///				They are the picture files underneath the base folder.
	///		</remark>
	/// </summary>
	public class GalleryGrid
	{
		private HttpResponse response;

		//
		//  private variables for properties
		//
		private string tableHtml;
		private string tableHeading;
		private string tableFooting;
		private string tableMiddle;
		private int cols;
		private Color color1;
		private Color color2;
		private bool alternative;

		//
		//  default values for properties
		//
		private const string TABLEHTML_DEFAULT = "<table width=95% cellpadding=5 cellspacing=0 border=0>";
		private const string TABLEHEADING_DEFAULT = "";
		private const string TABLEFOOTING_DEFAULT = "";
		private const string TABLEMIDDLE_DEFAULT = "";
		private const int COLS_DEFAULT = 5;
		private const bool ALTERNATIVE_DEFAULT = false;

		//
		//  default system folder for storing thumbnails
		//		Thumbnails are the small pictures for each picture files. GalleryNET will generate thumbnails automatically.
		//		The thumbail folder is put under the current album folder and named "_thumbnails".
		//		And the thumbail pictures are named after each picture's name with a "_" in the beginning.
		//
		private const string THUMBNAILFOLDER_DEFAULTNAME = "_thumbnails";

		/// <summary>
		///		The HTML code for starting the TABLE
		/// </summary>
		public string TableHtml
		{
			get {return tableHtml;}
			set {tableHtml = value;}
		}

		/// <summary>
		///		The HTML code for the first TR. 
		///		<remark>The column in this TR must be identical to Cols property</remark>
		/// </summary>
		public string TableHeading
		{
			get { return tableHeading;}
			set { tableHeading = value;}
		}

		/// <summary>
		///		The HTML code for the last TR.
		///		<remark>The column in this TR must be identical to Cols property</remark>
		/// </summary>
		public string TableFooting
		{
			get { return tableFooting;}
			set { tableFooting = value;}
		}

		/// <summary>
		///		The HTML code for the middle TR, which separate showing folders part and showing albums part
		///		<remark>The column in this TR must be identical to Cols property</remark>
		/// </summary>
		public string TableMiddle
		{
			get {return tableMiddle;}
			set { tableMiddle = value;}
		}

		/// <summary>
		///		Gets and sets the number of column in this table.
		/// </summary>
		public int Cols
		{
			get {return cols;}
			set {cols=value;}
		}

		/// <summary>
		///		Gets and sets whether the color of each row in this TABLE is alternative
		/// </summary>
		public bool Alternative
		{
			get {return alternative;}
			set {alternative=value;}
		}

		/// <summary>
		///		Gets and sets the default background color for each row
		/// </summary>
		public Color Color1
		{
			get {return color1;}
			set {color1=value;}
		}

		/// <summary>
		///		Gets and sets the background color for alternative color row
		/// </summary>
		public Color Color2
		{
			get {return color2;}
			set {color2=value;}
		}

		/// <summary>
		///		Initialize a new instance of GalleryGrid
		/// </summary>
		/// <param name="resp">HttpResponse object is used to output the HTML</param>
		public GalleryGrid(HttpResponse resp)
		{
			tableHtml = TABLEHTML_DEFAULT;
			tableHeading = TABLEHEADING_DEFAULT;
			tableFooting = TABLEFOOTING_DEFAULT;
			tableMiddle = TABLEMIDDLE_DEFAULT;
			cols = COLS_DEFAULT;
			alternative = ALTERNATIVE_DEFAULT;
			color1 = Color.Empty;
			color2 = Color.Empty;

			response = resp;
		}

		/// <summary>
		/// Display subfolders and picture files under the specified folder
		/// </summary>
		/// <param name="gridType">Specifies the show type in this GalleryGrid.</param>
		/// <param name="addonpath">
		///		The addonPath is specified as the rest part after [Album] directory
		///		<remark>
		///		For example, the base album folder name is [Album]. There is 
		///		a folder named [Girl] under the base. And, under the [Girl] folder there are
		///		three picture files; they are 1.jpg, 2.jpg, 3.jpg. To show 2.jpg on webpage,
		///		pass two parameters into this function, one is "2.jpg", specifies the filename
		///		to display; another is "Girl", specifies the addonPath.
		///		
		///		Regularly, the picture file name should be 
		///		[Base Folder Name]\[Sub Folder Name1]\[Sub Folder Name2]\[...]\FileName
		///		
		///		the addonPath is the section that deletes "[Base Folder Name]\" and "\FileName"
		///		</remark>
		/// </param>
		public void Show(GridType gridType, string addonpath)
		{

			// to do:
			// 1. ensure this addonpath dosen't containt "_thumbnail", which is a system folder
			// 2. show subfolders under this specified folder with TemplateShowFolder
			// 3. show album (pictures) under this specified folder

			if (addonpath.ToLower().IndexOf(THUMBNAILFOLDER_DEFAULTNAME)>0) return;

			string physicalPath = ApplicationConfiguration.PhysicalPath;
			string virtualPath = ApplicationConfiguration.VirtualPath;
			physicalPath = physicalPath + addonpath;
			virtualPath = virtualPath + addonpath;

			// begin the grid
			response.Write(TableHtml);

			// if TableHeader exists, write it
			if (TableHeading!="") response.Write(TableHeading);

			// counter for the writing column number
			int count = 0;
			// width for each <TD>, the value is on percent, divided by Cols
			int width = 100/Cols;

			if (gridType <= GridType.ShowFolder)
			{
				// create a DirectoryInfo object for the specified folder
				// get all subfolders under this folder
				DirectoryInfo currentDir = new DirectoryInfo(physicalPath);

				if (!currentDir.Exists)
				{
					response.Write("We do not show " + addonpath + " any more!");
					return;
				}

				DirectoryInfo[] subFolders = currentDir.GetDirectories();
				if (subFolders.Length>0) 
				{
					// filter the subfolders those name are "_thumbnails"
					for (int i=0;i<subFolders.Length;i++)
					{
						if (subFolders[i].FullName.ToLower().IndexOf(THUMBNAILFOLDER_DEFAULTNAME)<0)
						{
							WriteTRStart(count);

							response.Write("<td width='" + width +"%'>");
							Template.Builder.Translate(ApplicationConfiguration.TemplateShowFolder, subFolders[i],
								addonpath, response);
							response.Write("</td>");

							WriteTREnd(ref count);
						}
					}
				}
			}

			if (gridType == GridType.ShowAll || gridType == GridType.ShowAlbum) 
			{
				// after writing the subfolders, write TableMiddle if TableMiddle exists
				if (TableMiddle!="") response.Write(TableMiddle);
				// reset counter
				count = 0;

				// retrieve all picture files under this folder
				Thumbnails thumbs = new Thumbnails(physicalPath, virtualPath,
					ApplicationConfiguration.ThumbWidth,ApplicationConfiguration.ThumbHeight);
				GalleryInfo[] gi = thumbs.GetGalleryInfo();

				// if there is no picture, exit it
				if (gi==null)
				{
					// close grid firstly
					response.Write("</table>");
					return;
				}

				for (int i=0;i<gi.Length ;i++)
				{
					WriteTRStart(count);

					response.Write("<td width='" + width +"%'>");
					Template.Builder.Translate(ApplicationConfiguration.TemplateShowAlbum,gi[i],addonpath, response);
					response.Write("</td>");

					WriteTREnd(ref count);
				}
			}

			response.Write(TableFooting);

			// close the grid
			response.Write("</table>");
		}

		/// <summary>
		/// Write the start of TR tag
		/// </summary>
		/// <param name="count">the current column</param>
		private void WriteTRStart(int count)
		{
			if (count == 0)	
			{
				response.Write("<tr ");

				if (Alternative && (count % 2)==1)
				{
					if (Color2!=Color.Empty) response.Write("bgcolor=" + Color2.Name);
				}
				else
					if (Color1!=Color.Empty) response.Write("bgcolor=" + Color1.Name);

				response.Write(">");
			}
		}

		/// <summary>
		/// Write the end of TR tag
		/// </summary>
		/// <param name="count">the current column</param>
		private void WriteTREnd(ref int count)
		{
			if (count>=Cols-1)
			{
				response.Write("</tr>");
				count=0;
			}
			else
				count++;
		}
	}
}
