using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using angGoGo.GalleryNET.GalleryData;

namespace angGoGo.GalleryNET.Core
{
	/// <summary>
	/// Create a thumbnails for a specified folder
	/// </summary>
	public class Thumbnails
	{
		// the physical directory
		private string dpath="";
		// the thumbnail physical directory
		private string tpath = "";
		// the virtual directory name
		private string vipath="";

		// the DirectoryInfo for the specified directory
		private DirectoryInfo dDir;
		// the DirectoryInfo for the thumbnail directory
		private DirectoryInfo tDir;

		// the thumbnail width and height
		private int tWidth,tHeight;

		/// <summary>
		/// Create a folder that contains thumbnail images for all images files under the specified folder
		/// </summary>
		public void CreateThumbnail()
		{
			FileInfo[] dirFiles;
			Bitmap tmpBitmap;
			Image tmpImage;
			SizeF sizef;

			// firstly, delete all existed thumbnail picture files
			dirFiles = tDir.GetFiles(".jp*");
			if (dirFiles.Length > 0)
			{
				foreach (FileInfo fileinfo in dirFiles)
				{
					fileinfo.Delete();
				}
			}

			//Get only jpeg image files under the current folder
			dirFiles = dDir.GetFiles("*.jp*");

			if (dirFiles.Length >0) 
			{
				string extFile="";
				foreach (FileInfo fileinfo in dirFiles)
				{
					extFile = fileinfo.Extension ;
					tmpBitmap = new Bitmap(fileinfo.FullName);
					sizef = BitmapHelper.ScaleSize(tmpBitmap,tWidth,tHeight);

					tmpImage = BitmapHelper.GetThumbnail(tmpBitmap,(int)sizef.Width ,(int)sizef.Height);

					if (extFile.ToLower().Equals(".jpeg") || extFile.ToLower().Equals(".jpg"))
					{
						BitmapHelper.SaveJpeg(new Bitmap(tmpImage), tpath + "\\_" + fileinfo.Name,75);
					}
					else
					{
						tmpImage.Save(tpath + "\\_" + fileinfo.Name,System.Drawing.Imaging.ImageFormat.Gif);
					}
				}
			}
		}

		/// <summary>
		/// Initialize a new instance of Thumbnails with the specified physical directory, virtual directory, thumbnail image width and height
		/// </summary>
		/// <param name="path">Specified physical directory</param>
		/// <param name="vpath">Specified virtual directory</param>
		/// <param name="twidth">Thumbnail image width</param>
		/// <param name="theight">Thumbnail image height</param>
		public Thumbnails(string path,string vpath,int twidth,int theight)
		{
			dpath = path;
			vipath = vpath;
			tpath = dpath + "\\_thumbnails";
			dDir=new DirectoryInfo(dpath);

			// if the physical directory doesn't exist, throw the exception
			if (!dDir.Exists) throw new GalleryNETException("Source folder doesnot exist!");

			// if the thumbnail image size is smaller than zero, throw the exception
			if (twidth<0 || theight<0) throw new GalleryNETException("Thumbnail size must be larger than zero!");

			// create a DirectoryInfo object for the thumbnail directory
			tDir = new DirectoryInfo(tpath);
			tWidth = twidth;
			tHeight=theight;

			// if the thumbnail physical directory doesn't exist, create it
			// if the creation fails, throw the exception
			if (Count>0) 
			{
				if (!tDir.Exists) 
				{
					try 
					{
						tDir.Create();
					}
					catch (Exception ex)
					{
						throw new GalleryNETException(ex.Message);
					}
				}

				// if the file number under two folders are not equal, create the thumbnail again.
				if (Count!=ThumbnailCount) 
				{
					CreateThumbnail();
				}
			}
		}

		/// <summary>
		/// Gets the GalleryInfo objects for the specified directory
		/// </summary>
		/// <returns></returns>
		public GalleryInfo[] GetGalleryInfo()
		{
			if (Count<=0) return null;

			FileInfo[] dirFiles;
			dirFiles = dDir.GetFiles("*.jp*");
			Bitmap tmpBitmap;
			AlbumInfo ai=new AlbumInfo();
			SizeF sizef ;

			GalleryInfo[] gi=new GalleryInfo[dirFiles.Length];

			int i=0;

			foreach (FileInfo fileinfo in dirFiles) 
			{
				if (fileinfo.Extension.ToLower().Equals(".jpeg") ||
					fileinfo.Extension.ToLower().Equals(".jpg") || 
					fileinfo.Extension.ToLower().Equals(".gif")) 
				{
					tmpBitmap = new Bitmap(fileinfo.FullName);

					sizef = BitmapHelper.ScaleSize(tmpBitmap,this.tWidth,tHeight);

					gi[i].FullName = fileinfo.FullName;
					gi[i].FileName = fileinfo.Name;
					gi[i].VirtualFileName = vipath + "\\" + fileinfo.Name;
					gi[i].ThumbName = vipath + "\\_thumbnails\\_" + fileinfo.Name;
					gi[i].Size = fileinfo.Length;
					gi[i].PixelFormat = tmpBitmap.PixelFormat.ToString();
					gi[i].Width = tmpBitmap.Width;
					gi[i].Height = tmpBitmap.Height;
					gi[i].ThumbWidth = (int)sizef.Width;
					gi[i].ThumbHeight = (int)sizef.Height;
					gi[i].ClickCount = ai.GetClick(gi[i].VirtualFileName);
					gi[i].CreateTime = fileinfo.CreationTime;
					i++;
				}
			}

			ai.Dispose();
			return gi;
		}

		/// <summary>
		///		Gets the number of all pictures files under this specified folder
		/// </summary>
		public int Count 
		{
			get {return dDir.GetFiles("*.jp*").Length;}
		}

		/// <summary>
		///		Gets the number of thumbnail picture files under this specified folder
		/// </summary>
		public int ThumbnailCount
		{
			get {return tDir.GetFiles("*.jp*").Length;}
		}

	}


}
