namespace angGoGo.GalleryNET.SystemFramework
{
	using System;
	using System.Web;
	using System.Collections;
	using System.Diagnostics;
	using System.Configuration;
	using System.Xml;
	using System.Collections.Specialized;

	/// <summary>
	///     Standard configuration settings to enable tracing and logging
	///     with the ApplicationLog class.
	///     <remarks> 
	///         An application can use this class as a model for 
	///         adding additional settings to a Web.Config file.
	///         Special Considerations:
	///         The OnApplicationStart function in this class must be called
	///         from the Application_OnStart event in Global.asax. This is
	///         currently used to determine the path of the application,
	///         the HttpContext object is passed it to enable the app
	///         to read other settings in the future, and to minimize the code
	///         in global.asax. 
	///         <example>
	///             The global.asax file should be similar to the following code:
	///             <code>
	///                 <%@ Import Namespace="Duwamish7.SystemFramework"  %>
	///                 <script language="c#" runat="SERVER">
	///                     void Application_OnStart()
	///                     {
	///                         ApplicationConfiguration.OnApplicationStart(Context);
	///                     }
	///                 </script>
	///             </code>
	///         </example>
	///     </remarks>
	/// </summary>
	public class ApplicationConfiguration : IConfigurationSectionHandler
	{
		//
		// Constant values for all of the SystemFramework standard settings
		//
		// administration settings names
		private const String TRACING_ENABLED = "SystemFramework.Tracing.Enabled";
		private const String TRACING_TRACEFILE = "SystemFramework.Tracing.TraceFile";

		//private const String ADMIN_NAME = "SystemFramework.Admin.Name";
		//private const String ADMIN_PASS = "SystemFramework.Admin.Password";

		private const String SETTING_FOLDER = "SystemFramework.Directory.Setting";
		private const String TEMPLATE_SHOWFOLDER = "SystemFramework.Template.ShowFolder";
		private const String TEMPLATE_SHOWALBUM = "SystemFramework.Template.ShowAlbum";
		private const String TEMPLATE_SHOWFILE = "SystemFramework.Template.ShowFile";

		// album settings names
		private const String PHYSICAL_PATH = "SystemFramework.Directory.PhysicalPath";
		private const String VIRTUAL_PATH = "SystemFramework.Directory.VirutalPath";
		private const String THUMB_WIDTH = "SystemFramework.Thumbnail.Width";
		private const String THUMB_HEIGHT = "SystemFramework.Thumbnail.Height";
		private const String CLICK_ENABLED = "SystemFramework.ClickCount.Enabled";
		private const String CLICK_LOGFILE = "SystemFramework.ClickCount.LogFile";
        
		//
		// Static member variables. These contain the application settings
		//   from Config.Web, or the default values.
		//
		private static bool tracingEnabled;
		private static String tracingTraceFile;

		//private static String adminName;
		//private static String adminPass;
		private static String settingFolder;
		private static String templateShowFolder;
		private static String templateShowAlbum;
		private static String templateShowFile;
		private static String physicalPath;
		private static String virtualPath;
		private static int thumbWidth;
		private static int thumbHeight;
		private static bool clickEnabled;
		private static string clickLogFile;
        
		//
		// The root directory of the application. Established in the
		//   OnApplicationStart callback form Global.asax.
		//
		private static String appRoot;
        
		//
		// Constant values for all of the default settings.
		//
		private const bool TRACING_ENABLED_DEFAULT = true;
		private const String TRACING_TRACEFILE_DEFAULT = "GalleryNETTrace.txt";

		//private const String ADMIN_NAME_DEFAULT = "admin";
		//private const String ADMIN_PASS_DEFAULT = "";
		private const String SETTING_FOLDER_DEFAULT = "config";
		private const String TEMPLATE_SHOWFOLDER_DEFAULT = "showfolder.html";
		private const String TEMPLATE_SHOWALBUM_DEFAULT = "showalbum.html";
		private const String TEMPLATE_SHOWFILE_DEFAULT = "showfile.html";

		// album settings names
		//private const String PHYSICAL_PATH_DEFAULT = ""; //default of physical path will be the appRoot
		private const String VIRTUAL_PATH_DEFAULT = "album";
		private const int THUMB_WIDTH_DEFAULT = 100;
		private const int THUMB_HEIGHT_DEFAULT = 100;
		private const bool CLICK_ENABLED_DEFAULT = true;
		private const String CLICK_LOGFILE_DEFAULT = "GalleryNETLog.txt";

		/// <summary>
		///     Called from OnApplicationStart to initialize settings from
		///     the Web.Config file(s). 
		///     <remarks>
		///         The app domain will restart if settings change, so there is 
		///         no reason to read these values more than once. This funtion
		///         uses the NameValueSectionHandler base class to generate a 
		///         hashtablefrom the XML, which is then used to store the current
		///         settings.  Because all settings are read here, we do not actually 
		///         store the generated hashtable object for later retrieval by
		///         Context.GetConfig. The application should use the accessor
		///         functions directly.
		///     </remarks>
		///     <param name="parent">An object created by processing a section 
		///         with this name in a Config.Web file in a parent directory.
		///     </param>
		///     <param name="configContext">The config's context.</param>
		///     <param name="section">The section to be read.</param>
		///     <retvalue>
		///		    <para>
		///             A ConfigOutput object: which we leave empty because all settings
		///             are stored at this point.
		///		    </para>
		///		    <para>
		///             null:  if there was an error.
		///		    </para>
		///	    </retvalue>
		/// </summary>
		public Object Create(Object parent, object configContext, XmlNode section)
		{
                    
			NameValueCollection settings;
                    
			try
			{
				NameValueSectionHandler baseHandler = new NameValueSectionHandler();
				settings = (NameValueCollection)baseHandler.Create(parent, configContext, section);
			}
			catch
			{
				settings = null;
			}
        
			if (settings == null)
			{
				tracingEnabled = TRACING_ENABLED_DEFAULT;
				tracingTraceFile = TRACING_TRACEFILE_DEFAULT;

				//adminName = ADMIN_NAME_DEFAULT;
				//adminPass = ADMIN_PASS_DEFAULT;

				settingFolder = SETTING_FOLDER_DEFAULT;
				templateShowFolder = TEMPLATE_SHOWFOLDER_DEFAULT;
				templateShowAlbum = TEMPLATE_SHOWALBUM_DEFAULT;
				templateShowFile = TEMPLATE_SHOWFILE_DEFAULT;
				physicalPath = appRoot;
				virtualPath = VIRTUAL_PATH_DEFAULT;
				thumbWidth = THUMB_WIDTH_DEFAULT;
				thumbHeight = THUMB_HEIGHT_DEFAULT;
				clickEnabled = CLICK_ENABLED_DEFAULT;
			}
			else
			{
				tracingEnabled = ReadSetting(settings, TRACING_ENABLED, TRACING_ENABLED_DEFAULT);
				tracingTraceFile = ReadSetting(settings, TRACING_TRACEFILE, TRACING_TRACEFILE_DEFAULT);

				//adminName = ReadSetting(settings, ADMIN_NAME, ADMIN_NAME_DEFAULT);
				//adminPass = ReadSetting(settings, ADMIN_PASS, ADMIN_PASS_DEFAULT);

				settingFolder = ReadSetting(settings, SETTING_FOLDER, SETTING_FOLDER_DEFAULT);
				templateShowFolder = ReadSetting(settings, TEMPLATE_SHOWFOLDER, TEMPLATE_SHOWFOLDER_DEFAULT);
				templateShowAlbum = ReadSetting(settings, TEMPLATE_SHOWALBUM, TEMPLATE_SHOWALBUM_DEFAULT);
				templateShowFile = ReadSetting(settings, TEMPLATE_SHOWFILE, TEMPLATE_SHOWFILE_DEFAULT);
				physicalPath = ReadSetting(settings, PHYSICAL_PATH, appRoot);
				virtualPath = ReadSetting(settings, VIRTUAL_PATH, VIRTUAL_PATH_DEFAULT);
				thumbWidth = ReadSetting(settings, THUMB_WIDTH, THUMB_WIDTH_DEFAULT);
				thumbHeight = ReadSetting(settings, THUMB_HEIGHT, THUMB_HEIGHT_DEFAULT);
				clickEnabled = ReadSetting(settings, CLICK_ENABLED, CLICK_ENABLED_DEFAULT);
				clickLogFile = ReadSetting(settings, CLICK_LOGFILE, CLICK_LOGFILE_DEFAULT);
			}

			return null;
		}

		/// <summary>
		///     String version of ReadSetting.
		///     <remarks>
		///         Reads a setting from a hashtable and converts it to the correct
		///         type. One of these functions is provided for each type
		///         expected in the hash table. These are public so that other
		///         classes don't have to duplicate them to read settings from
		///         a hash table.
		///     </remarks>
		///     <param name="settings">The Hashtable to read from.</param>
		///     <param name="key">A key for the value in the Hashtable.</param>
		///     <param name="defaultValue">The default value if the item is not found.</param>
		///     <retvalue>
		///		    <para>value: from the hash table</para>
		///         <para>
		///             default: if the item is not in the table or cannot be case to the expected type.
		///		    </para>
		///	    </retvalue>
		/// </summary>
		public static String ReadSetting(NameValueCollection settings, String key, String defaultValue)
		{
			try
			{
				Object setting = settings[key];
                
				return (setting == null) ? defaultValue : (String)setting;
			}
			catch
			{
				return defaultValue;
			}
		}
        
		/// <summary>
		///     Boolean version of ReadSetting.
		///     <remarks>
		///         Reads a setting from a hashtable and converts it to the correct
		///         type. One of these functions is provided for each type
		///         expected in the hash table. These are public so that other
		///         classes don't have to duplicate them to read settings from
		///         a hash table.
		///     </remarks>
		///     <param name="settings">The Hashtable to read from.</param>
		///     <param name="key">A key for the value in the Hashtable.</param>
		///     <param name="defaultValue">The default value if the item is not found.</param>
		///     <retvalue>
		///		    <para>value: from the hash table</para>
		///         <para>
		///             default: if the item is not in the table or cannot be case to the expected type.
		///		    </para>
		///	    </retvalue>
		/// </summary>
		public static bool ReadSetting(NameValueCollection settings, String key, bool defaultValue)
		{
			try
			{
				Object setting = settings[key];
                
				return (setting == null) ? defaultValue : Convert.ToBoolean((String)setting);
			}
			catch
			{
				return defaultValue;
			}
		}
        
		/// <summary>
		///     int version of ReadSetting.
		///     <remarks>
		///         Reads a setting from a hashtable and converts it to the correct
		///         type. One of these functions is provided for each type
		///         expected in the hash table. These are public so that other
		///         classes don't have to duplicate them to read settings from
		///         a hash table.
		///     </remarks>
		///     <param name="settings">The Hashtable to read from.</param>
		///     <param name="key">A key for the value in the Hashtable.</param>
		///     <param name="defaultValue">The default value if the item is not found.</param>
		///     <retvalue>
		///		    <para>value: from the hash table</para>
		///         <para>
		///             default: if the item is not in the table or cannot be case to the expected type.
		///		    </para>
		///	    </retvalue>
		/// </summary>
		public static int ReadSetting(NameValueCollection settings, String key, int defaultValue)
		{
			try
			{
				Object setting = settings[key];
                
				return (setting == null) ? defaultValue : Convert.ToInt32((String)setting);
			}
			catch
			{
				return defaultValue;
			}
		}
        
		/// <summary>
		///     TraceLevel version of ReadSetting.
		///     <remarks>
		///         Reads a setting from a hashtable and converts it to the correct
		///         type. One of these functions is provided for each type
		///         expected in the hash table. These are public so that other
		///         classes don't have to duplicate them to read settings from
		///         a hash table.
		///     </remarks>
		///     <param name="settings">The Hashtable to read from.</param>
		///     <param name="key">A key for the value in the Hashtable.</param>
		///     <param name="defaultValue">The default value if the item is not found.</param>
		///     <retvalue>
		///		    <para>value: from the hash table</para>
		///         <para>
		///             default: if the item is not in the table or cannot be case to the expected type.
		///		    </para>
		///	    </retvalue>
		/// </summary>
		public static TraceLevel ReadSetting(NameValueCollection settings, String key, TraceLevel defaultValue)
		{
			try
			{
				Object setting = settings[key];
                
				return (setting == null) ? defaultValue : (TraceLevel)Convert.ToInt32((String)setting);
			}
			catch
			{
				return defaultValue;
			}
		}
        
		/// <summary>
		///     Function to be called by Application_OnStart as described in the
		///     class description. Initializes the application root.
		///     <param name="myAppPath">The path of the running application.</param>
		/// </summary>
		public static void OnApplicationStart(String myAppPath)
		{
			appRoot = myAppPath;
			System.Configuration.ConfigurationSettings.GetConfig("ApplicationConfiguration");
		}

		/*
		/// <summary>
		///    Validate the administrator name and password.
		/// </summary>
		/// <param name="name">The name of administrator.</param>
		/// <param name="password">The password of administrator.</param>
		/// <returns></returns>
		public static bool ValidAdmin(String name,String password)
		{
			if (name==adminName) 
			{
				if (password==String.Empty && adminPass == String.Empty)
					return true;

			}

			//any other cases return false
			return false;
		}
		*/

		#region properties
		/// <value>
		///     Property TracingEnabled is used to get the configuration setting, defaulting to False on error.
		/// </value>
		public static bool TracingEnabled
		{
			get	
			{
				return tracingEnabled;	
			}
		}
        
		/// <value>
		///     Property TracingTraceFile is used to get the full path name to the file that contains trace 
		///     settings, defaults to ApplicationTrace.txt.
		/// </value>
		public static String TracingTraceFile
		{
			get	
			{
				return tracingTraceFile;	
			}
		}
        
		/// <value>
		///     Property TracingTraceFile is used to get the highest logging level that should be ritten to 
		///     the tracing file, defaults to TraceLevel.Verbose (however, TracingEnabled defaults
		///     to False, so you have to turn it on explicitly).
		/// </value>
//		public static TraceLevel TracingTraceLevel
//		{
//			get	
//			{
//				return tracingTraceLevel;	
//			}
//		}

		/// <summary>
		///		Property SettingFolder is used to get the physical path to the folder under which all system
		///		files are put, including template files and tracing log file
		/// </summary>
		public static string SettingFolder
		{
			get	
			{
				if (settingFolder.LastIndexOf('/')<0 || settingFolder.LastIndexOf('\\')<0)
					return settingFolder + "/";
				else
					return settingFolder;
			}
		}

		/// <summary>
		///		Property TemplateShowFolder is used to get the template name to show folder list. It will
		///		be combined with SettingFolder property when needed.
		/// </summary>
		public static string TemplateShowFolder
		{
			get	
			{
				return SettingFolder + templateShowFolder;	
			}
		}

		/// <summary>
		///		Property TemplateShowAlbum is used to get the template name to show album list. It will
		///		be combined with SettingFolder property when needed.
		/// </summary>
		public static string TemplateShowAlbum
		{
			get 
			{
				return SettingFolder + templateShowAlbum;
			}
		}

		/// <summary>
		///		Property TemplateShowFile is used to get the template name to show file. It will
		///		be combined with SettingFolder property when needed.
		/// </summary>
		public static string TemplateShowFile
		{
			get 
			{
				return SettingFolder + templateShowFile;	
			}
		}

		/// <summary>
		///		Property PhysicalPath is used to get the physical path to the root of album.
		/// </summary>
		public static string PhysicalPath
		{
			get 
			{
				if (physicalPath.LastIndexOf('/')<0 || physicalPath.LastIndexOf('\\')<0)
					return physicalPath + "/";
				else
					return physicalPath;
			}
		}

		/// <summary>
		///		Property VirtualPath is used to get the virutal path to the root of album.
		///		Virtual path specifies a mapping between the virtual path in a URL and the physical path on the device.
		/// </summary>
		public static string VirtualPath
		{
			get 
			{ 
				if (virtualPath.LastIndexOf('/')<=0 || virtualPath.LastIndexOf('\\')<=0)
					return virtualPath + "/";
				else
					return virtualPath;
			}
		}

		/// <summary>
		///		Property ThumbWidth is used to get the thumbnail width.
		/// </summary>
		public static int ThumbWidth
		{
			get 
			{
				return thumbWidth;
			}
		}

		/// <summary>
		///		Property ThumbHeight is used to get the thumbnail height.
		/// </summary>
		public static int ThumbHeight
		{
			get 
			{
				return thumbHeight;
			}
		}

		/// <summary>
		///		Property ClickEnabled is used to get the whether log the click count, which specifies 
		///		counting the image click times.
		/// </summary>
		public static bool ClickEnabled
		{
			get 
			{
				return clickEnabled;
			}
		}

		/// <summary>
		///		Property ClickLogFile is used to get the filename to log click count.
		/// </summary>
		public static string ClickLogFile
		{
			get
			{
				return clickLogFile;
			}
		}


		#endregion

	}
}
